@@ -1,10 +1,11 @@
-# SPDX-License-Identifier: Apache-2.0.
+# Copyright 2018-2022 contributors to the OpenLineage project
+# SPDX-License-Identifier: Apache-2.0
 from typing import Dict, List, Optional
 
 import attr
 
 from openlineage.client.constants import DEFAULT_PRODUCER
-
+from openlineage.client.utils import RedactMixin
 
 SCHEMA_URI = "https://raw.githubusercontent.com/OpenLineage/OpenLineage/main/spec/OpenLineage.json"
 
@@ -17,10 +18,13 @@ def set_producer(producer):
 
 
 @attr.s
-class BaseFacet:
+class BaseFacet(RedactMixin):
     _producer: str = attr.ib(init=False)
     _schemaURL: str = attr.ib(init=False)
 
+    _base_skip_redact: List[str] = ['_producer', '_schemaURL']
+    _additional_skip_redact: List[str] = []
+
     def __attrs_post_init__(self):
         self._producer = PRODUCER
         self._schemaURL = self._get_schema()
@@ -29,12 +33,18 @@ class BaseFacet:
     def _get_schema() -> str:
         return SCHEMA_URI + "#/definitions/BaseFacet"
 
+    @property
+    def skip_redact(self):
+        return self._base_skip_redact + self._additional_skip_redact
+
 
 @attr.s
 class NominalTimeRunFacet(BaseFacet):
     nominalStartTime: str = attr.ib()
     nominalEndTime: Optional[str] = attr.ib(default=None)
 
+    _additional_skip_redact: List[str] = ['nominalStartTime', 'nominalEndTime']
+
     @staticmethod
     def _get_schema() -> str:
         return SCHEMA_URI + "#/definitions/NominalTimeRunFacet"
@@ -45,6 +55,8 @@ class ParentRunFacet(BaseFacet):
     run: Dict = attr.ib()
     job: Dict = attr.ib()
 
+    _additional_skip_redact: List[str] = ['job', 'run']
+
     @classmethod
     def create(cls, runId: str, namespace: str, name: str):
         return cls(
@@ -76,6 +88,8 @@ class SourceCodeLocationJobFacet(BaseFacet):
     type: str = attr.ib()
     url: str = attr.ib()
 
+    _additional_skip_redact: List[str] = ['type', 'url']
+
     @staticmethod
     def _get_schema() -> str:
         return SCHEMA_URI + "#/definitions/SourceCodeLocationJobFacet"
@@ -100,11 +114,13 @@ class DocumentationDatasetFacet(BaseFacet):
 
 
 @attr.s
-class SchemaField:
+class SchemaField(RedactMixin):
     name: str = attr.ib()
     type: str = attr.ib()
     description: Optional[str] = attr.ib(default=None)
 
+    _do_not_redact = ['name', 'type']
+
 
 @attr.s
 class SchemaDatasetFacet(BaseFacet):
@@ -120,6 +136,8 @@ class DataSourceDatasetFacet(BaseFacet):
     name: str = attr.ib()
     uri: str = attr.ib()
 
+    _additional_skip_redact: List[str] = ['name', 'uri']
+
     @staticmethod
     def _get_schema() -> str:
         return SCHEMA_URI + "#/definitions/DataSourceDatasetFacet"
@@ -130,6 +148,8 @@ class OutputStatisticsOutputDatasetFacet(BaseFacet):
     rowCount: int = attr.ib()
     size: Optional[int] = attr.ib(default=None)
 
+    _additional_skip_redact: List[str] = ['rowCount', 'size']
+
     @staticmethod
     def _get_schema() -> str:
         return SCHEMA_URI + "#/definitions/OutputStatisticsOutputDatasetFacet"
@@ -158,11 +178,13 @@ class DataQualityMetricsInputDatasetFacet(BaseFacet):
 
 
 @attr.s
-class Assertion:
+class Assertion(RedactMixin):
     assertion: str = attr.ib()
     success: bool = attr.ib()
     column: Optional[str] = attr.ib(default=None)
 
+    _skip_redact: List[str] = ['column']
+
 
 @attr.s
 class DataQualityAssertionsDatasetFacet(BaseFacet):
@@ -184,6 +206,8 @@ class SourceCodeJobFacet(BaseFacet):
     language: str = attr.ib()  # language that the code was written in
     source: str = attr.ib()  # source code text
 
+    _additional_skip_redact: List[str] = ['language']
+
     @staticmethod
     def _get_schema() -> str:
         return SCHEMA_URI + "#/definitions/SourceCodeJobFacet"
@@ -193,3 +217,19 @@ class SourceCodeJobFacet(BaseFacet):
 class ExternalQueryRunFacet(BaseFacet):
     externalQueryId: str = attr.ib()
     source: str = attr.ib()
+
+
+@attr.s
+class ErrorMessageRunFacet(BaseFacet):
+    """
+    This facet represents an error message that was the result of a job run
+    """
+    message: str = attr.ib()
+    programmingLanguage: str = attr.ib()
+    stackTrace: Optional[str] = attr.ib(default=None)
+
+    _additional_skip_redact: List[str] = ['programmingLanguage']
+
+    @staticmethod
+    def _get_schema() -> str:
+        return SCHEMA_URI + "#/definitions/ErrorMessageRunFacet"
